@@ -6,7 +6,7 @@ AUTHOR: MICHAEL SILVERSTEIN
 EMAIL: michael.silverstein4@gmail.com
 """
 import argparse
-import random, os
+import os
 from .utils import Project, default_nreads, default_mal, default_mae
 from .viz import Viz
 from argparse import ArgumentParser, RawTextHelpFormatter
@@ -43,7 +43,7 @@ def truncPipeline(filepaths: list, fwd_format: str, rev_format = None, mal=defau
 
     # 2) Subsample
     print('\t\t**** RUNNING MINEER ALGORITHM ****')
-    print('Running minEER on a subset of %d reads per direction...' % nreads)
+    print('Running minEER on until a subset of at least %d reads per direction...' % nreads)
     project.subsampleAll()
     print('Complete.')
     print()
